/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is NewsAlert code.
 *
 * The Initial Developer of the Original Code is 
 * Doug Turner
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Doug Turner <dougt@acm.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 *
  I use the following unfrozen interfaces:

  mozIJSSubScriptLoader

  nsICategoryManager
  nsIIOService
  nsIProfileStartupListener
  nsIScriptableInputStream
  nsIScriptableTimer
*/

const kDebug               = true;
const kNAEnabledPref       = "news_alert.enabled";

// These are the functions and variables which the news providers define
function parseNews(raw){}
var providerURL;
var providerName;
var providerInterval;


function toScriptableStream(input) {
    var SIStream = Components.Constructor('@mozilla.org/scriptableinputstream;1',
                                          'nsIScriptableInputStream', 'init');
    return new SIStream(input);
}

function openWindowWithContent(content) {
    
    var winWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].
        getService(Components.interfaces.nsIWindowWatcher);
    
    var window = winWatcher.openWindow(winWatcher.activeWindow, 
                                       "about:blank", 
                                       "_blank", 
                                       "chrome,all,dialog=yes,resizable,scrollbars=no", 
                                       null);

    var document = window.document;
    document.write(content);
    document.close();

    window.sizeToContent();
    debug("window opened.");       
}


////////////////////////////////////////////////////////////////////////
//
//   newsAlertContext
//
////////////////////////////////////////////////////////////////////////
function newsAlertContext() {}

newsAlertContext.prototype = 
{
    
    get newsID()       { return this.mID; },
    set newsID(newval) { return this.mID = newval; },
    mID: -1,

    get newsBuffer()       { return this.mBuffer; },
    set newsBuffer(newval) { return this.mBuffer = newval; },
    mBuffer: "",

    QueryInterface: function(aIID)
    {
        if (!aIID.equals(Components.interfaces.newsAlertContext) &&
            !aIID.equals(Components.interfaces.nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        
        return this;
    },
};


////////////////////////////////////////////////////////////////////////
//
//   newsAlertProvider
//
////////////////////////////////////////////////////////////////////////

function newsAlertProvider() {}

newsAlertProvider.prototype = 
{
    mName:                 "", // Name of provider
    mLastNews:             "", // This is the string which we was last displayed to the users.
    mScriptURL:            "", // Local URL to script
    mURL:                  "", // URL to news provider
    mInterval:             0, // How often this provider should be called.
    mTicks:                0, // How many times has the timer passed.
}

////////////////////////////////////////////////////////////////////////
//
//   newsAlertNotifier
//
////////////////////////////////////////////////////////////////////////
var newsAlertNotifier = 
{    
    mTimer:  null, // need to hold on to timer ref
    
    mProviders: null, // this is the array of news alert provider that are 
    // supported.  This listing is ordered such that indexes 
    //between the array mSeen and the function parseNewsAlert
    // must be uniform. 
    
    onProfileStartup: function(aProfileName)
    {
        debug("onProfileStartup");
        
        var prefs  = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefBranch);

        try {
            if (!prefs.getBoolPref(kUNEnabledPref))
                return;
        }
        catch (ex){
            // not enabled, but this is a opt out system.
        }
        
        mProviders = new Array (0);
        
        // find all of the news providers and populate the mProviders array
        try {
            var dirServiceProvider = Components.classes["@mozilla.org/file/directory_service;1"]
                .getService(Components.interfaces.nsIDirectoryServiceProvider);
            
            var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                .getService(Components.interfaces.nsIIOService);

            var persistent = new Object();     
            var newsServiceDir = dirServiceProvider.getFile("ComsD", persistent);
            newsServiceDir.append("newsAlert");
            
            var directoryListing = newsServiceDir.directoryEntries;
            while (directoryListing.hasMoreElements()) { 
                var file = directoryListing.getNext().QueryInterface(Components.interfaces.nsIFile);
                var leafName = file.leafName;

                // we want to keep all of our binary pieces in one directory, so
                // we need to filter out the stuff that isn't a provider
                if (leafName == "newsAlert.js" || 
                    leafName == "newsAlert.xpt" )
                    continue;
                
                var provider  = new newsAlertProvider();
                provider.mScriptURL = ioService.newFileURI(file).spec;

                mProviders.push(provider);
                var providerID = mProviders.length - 1; // array is zero based.

                debug("News Providers: " + leafName + " at index " + providerID);

                // Load the provider to access provider data.
                this.loadNewsProvider(providerID);

                provider.mName      = providerName;
                provider.mURL       = providerURL;
                provider.mInterval  = providerInterval;
                
                // populate the last seen items.
                try { 
                    //news_alert.foo.mLast  TODO: Make this a list of the last 10 or so seen.
                    provider.mSeen = prefs.getCharPref("news_alert." + providerName + ".mLast"); // secs 
                }
                catch (ex) {}
            }

            const kIScriptableTimer = Components.interfaces.nsIScriptableTimer;
            // fire the timer which does the work.
            mTimer = Components.classes["@mozilla.org/timer;1"]
                .createInstance(kIScriptableTimer);

            mTimer.init(this, 
                        60*1000, // timer is every minute.     
                        kIScriptableTimer.PRIORITY_NORMAL,
                        kIScriptableTimer.TYPE_REPEATING_SLACK);
        }
        catch (ex) {
            debug("Loading news services failed: " + ex);
        }
    },
    
    observe: function(aSubject, aTopic, aData)
    {
        debug("observe: " + aTopic);
        
        // the only topic we should get is "timer-callback"
        this.checkForUpdate();

    },
    
    writeProviderToPrefs: function(id)
    {
        var prefs  = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefBranch);

        // TODO: support more than one mSeen.
        prefs.setCharPref("news_alert." + mProviders[id].mName + ".mLast", 
                          mProviders[id].mSeen);
    },

    loadNewsProvider: function(id)
    {
        debug("Loading new provider: " + id);
               
        try {
            var subscriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
            .createInstance(Components.interfaces.mozIJSSubScriptLoader);

            subscriptLoader.loadSubScript(mProviders[id].mScriptURL);
        }
        catch (ex) {
            debug("loadNewsProvider failed: " + id +" "+ ex);
        }
    },

    parseNewsAlert: function(id, raw)
    {
        this.loadNewsProvider(id)

        try {
            return parseNews(raw);
        }
        catch (ex) {
            debug("Parsing of news failed: " + ex);
        }
        return "";
    },

    checkForUpdate: function()
    {
        debug("checkForUpdate");
        
        var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
        var ticks;
        var index;

        for (index = 0; index < mProviders.length; index++) {

            ticks = mProviders[index].mTicks; 

            if (ticks > mProviders[index].mInterval) {
                mProviders[index].mTicks = ticks + 1;
                continue;
            }

            // this sets providerURL
            this.loadNewsProvider(index);

            var context = new newsAlertContext();
            context.newsID = index;
            context.newsBuffer = "";

            // create new url.  most news sites wont cache pages.  why bother.
            var channel = ioService.newChannel(providerURL, "", null);
            channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE | 
                                 Components.interfaces.nsIRequest.INHIBIT_CACHING ;
            
            channel.asyncOpen(this, context);
            debug("download initalized: " + mProviders[index].mName);    
        }
    },
    
    // nsIStreamListener interface
    onStartRequest: function(request, context) { 
        debug(request.name + " onStartRequest fired");
    },
    
    onStopRequest: function(request, context, status, errorMsg) {
        try {
            var id     = context.newsID;
            var string = context.newsBuffer;

            debug(request.name + " onStopRequest fired");
        
            var news = this.parseNewsAlert(id, string);
            
            if (news.length == 0)
                return;

            // TODO: add support for more than one mSeen.
            if (mProviders[id].mSeen == news) {
                debug("newsAlert seen already");
                return;
            }
            
            mProviders[id].mSeen = news;
            // go ahead and write it to preferences.
            this.writeProviderToPrefs(id);
            openWindowWithContent(news);
        }
        catch (ex) {
            debug("Exception during onStopRequest: " + ex);
        }
    },
    
    onDataAvailable: function(request, context, inStream, sourceOffset, count) {
        var sis = toScriptableStream(inStream);
        context = context.QueryInterface(Components.interfaces.newsAlertContext);
        
        var string = context.newsBuffer;
        string += sis.read(count);
        context.newsBuffer = string;
        sis.close();
    },
    
    QueryInterface: function(aIID)
    {
        if (!aIID.equals(Components.interfaces.nsIObserver) &&
            !aIID.equals(Components.interfaces.nsIProfileStartupListener) &&
            !aIID.equals(Components.interfaces.nsIStreamListener) &&
            !aIID.equals(Components.interfaces.nsIRequestObserver) &&
            !aIID.equals(Components.interfaces.nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        
        return this;
    }
}

////////////////////////////////////////////////////////////////////////
//
//   newsAlertNotifierModule : nsIModule
//
////////////////////////////////////////////////////////////////////////

var newsAlertNotifierModule = 
{
    mClassName:     "News Alert Notifier",
    mContractID:    "@mozilla.org/news-alert-notifier;1",
    mClassID:       Components.ID("a2af41c2-d2dc-448c-a091-e2b6f8370975"),
    
    getClassObject: function(aCompMgr, aCID, aIID)
    {
        if (!aCID.equals(this.mClassID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!aIID.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        
        return this.mFactory;
    },
    
    registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
    {
        if (kDebug)
        debug("*** Registering newsAlertNotifier (a JavaScript Module)\n");
        
        aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.registerFactoryLocation(this.mClassID, 
                                         this.mClassName,
                                         this.mContractID, 
                                         aFileSpec, 
                                         aLocation, 
                                         aType);
        
        // receive startup notification from the profile manager
        // (we get |createInstance()|d at startup-notification time)
        this.getCategoryManager().addCategoryEntry("profile-startup-category", 
                                                   this.mContractID, "", true, true);
    },
    
    unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
    {
        aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.unregisterFactoryLocation(this.mClassID, 
                                           aFileSpec);
        
        this.getCategoryManager().deleteCategoryEntry("profile-startup-category", 
                                                      this.mContractID, true);

        // deleteBranch
    },
    
    canUnload: function(aCompMgr)
    {
        return true;
    },
    
    getCategoryManager: function()
    {
        return Components.classes["@mozilla.org/categorymanager;1"].
        getService(Components.interfaces.nsICategoryManager);
    },
    
    //////////////////////////////////////////////////////////////////////
    //
    //   mFactory : nsIFactory
    //
    //////////////////////////////////////////////////////////////////////
    mFactory:
    {
        createInstance: function(aOuter, aIID)
        {
            if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
            if (!aIID.equals(Components.interfaces.nsIObserver) &&
                !aIID.equals(Components.interfaces.nsIProfileStartupListener) &&
                !aIID.equals(Components.interfaces.nsIStreamListener) &&
                !aIID.equals(Components.interfaces.nsIRequestObserver) &&
                !aIID.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;
            
            // return the singleton 
            return newsAlertNotifier.QueryInterface(aIID);
        },
        
        lockFactory: function(aLock)
        {
            // quiten warnings
        }
    }
};

function NSGetModule(aCompMgr, aFileSpec)
{
    return newsAlertNotifierModule;
}



////////////////////////////////////////////////////////////////////////
//
//   Debug helper
//
////////////////////////////////////////////////////////////////////////
if (!kDebug)
    debug = function(m) {};
else
    debug = function(m) {dump("\t-->: " + m + "\n");};

